//a lot of the calculation code adapted from Dead Reckoning work by Aaron Berk
//http://mbed.org/users/aberk/notebook/dead-reckoning/

#include "trtSettings.h"
#include "trtkernel644.c"
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include "spi_init.h"
#include "adxl345.h"
#include "lcd_lib.h"

//semaphore definesg
#define SEM_RX_ISR_SIGNAL 1
#define SEM_STRING_DONE 2 // user hit <enter>
// semaphore to protect shared variable
#define SEM_SHARED 3
#define SEM_LCD_SHARED 4

//#include "uart.h"
#include "trtUart.h"
#include "trtUart.c"

//SD includes
#include "mmc.c"
#include "ff.c"

//Noise Averaging
#define CALIBRATION_COUNT 15
#define STANDING 3 //# of times we have to read 0 to recognize no motion
#define NOISE_AVERAGE 4 //#of times raw acceleration data is averaged over to reduce noise
#define GAIN_FULL .03822 //3.9mg/LSB
#define ZNOISE_MIN -0.1911 //min noise threshold to count as meaningful data
#define ZNOISE_MAX 0.1911 
//Step Thresholds
#define ZSTEP_THRESH 8 //accel running average threshold to count as step
#define ZSTEP_THRESHbar -8
#define ZSTEP_1 800 //min time between same foot step to count as step
#define ZSTEP_2 400 //min time between opposite foot step to count as step
#define ZSTEP_1_FAST 400 //adjustment for faster steps
#define ZSTEP_2_FAST 200
//Pace Thresholds
#define JERK_THRESH 18 //jerk threshold to toggle between fast and slow steps 
#define SLOW_FREQ stepAvg < 1.8  //slow walk threshold of steps per second 
#define MEDIUM_FREQ stepAvg < 2.5  //medium walk threshold
#define FAST_FREQ stepAvg < 3.4  //fast walk threshold
#define SLOW_WALK .00055 //slow walk distance per step
#define MEDIUM_WALK	.00068 //medium walk
#define FAST_WALK .00078 //fast walk
#define RUN .00082 //very fast walk

//Button State Machine PIND.7
//active HIGH
#define BUTTON PIND & 0x80 
#define NOPUSH 1
#define MAYBE 2
#define PUSH 3
#define MAYBENO 4
#define IDLE 0
#define INIT 1
#define ACTIVE 2

//LCD constants
const uint8_t LCD_initialize[] PROGMEM = "Welcome \0";
const uint8_t LCD_time[] PROGMEM = "Time: 	\0";
const uint8_t LCD_step[] PROGMEM = "Step:           \0";
const uint8_t LCD_dist[] PROGMEM = "Distance:       \0";
const uint8_t LCD_avsp[] PROGMEM = "Speed:          \0";
const uint8_t LCD_calibrate[] PROGMEM = "Calibrating...\0";
const uint8_t LCD_file[] PROGMEM = "File Opened\0";
const uint8_t LCD_fail[] PROGMEM = "File Failed\0";
const uint8_t LCD_idle[] PROGMEM = "Press to start\0";
const uint8_t LCD_close[] PROGMEM = "File Closed\0";
uint8_t lcd_buffer[17];	

//trt variables
int args[2] ;
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//timer variables
volatile uint8_t integralCount;
volatile uint16_t sec, ms, total_ms, stepms1, stepms2, step2step, window;
uint8_t dt;

//state machine variables
volatile uint8_t btct, btstate, runstate;

//raw z-axis data
uint8_t zLSB, zMSB;
uint16_t zAccel1, zAccel2;

//averaged z-axis data
double zAccumulator1, zAccumulator2, zAcc1, zAcc2;

//variables for step frequency
double stepHz, pstep1, pstep2, pstep3, pstep4, stepAvg, currentSpeed, curSP;
 
//variables for jerk 
double jerk1, pj1, pj2, pj3, pj4, pj5, pj6, pj7, jerk1Avg;
double jerk2, pjj1, pjj2, pjj3, pjj4, pjj5, pjj6, pjj7, jerk2Avg;
uint16_t timeSmall, timeLarge;

//variables for running average of z-axis
double pz1, pz2, pz3, pz4, zAccRun1, zAccRun1bar;
double pzz1, pzz2, pzz3, pzz4, zAccRun2, zAccRun2bar;


double latdis, dy;
char sample;
uint16_t zBias1, zBias2;
uint16_t step;   
int calibrationCount; 
char led;

// SD Card flags & variables
// run_fl: 	1 if data acquisition has started, 0 if not
// SD_fl: 	1 if SD file was successfully opened and can be written to, 0 if not
// bt_fl:	1 if btstate has been pushed AND the push has not yet been processed by stepData().
//			0 if btstate is not pushed or push has already been processed only one push at a time is recorded. 
char run_fl, init_fl, SD_fl, bt_fl; 
FIL fil_obj;
FRESULT f_err_code;
DSTATUS status;
static FATFS FATFS_Obj;
unsigned int fileCounter; 
#define diskInterval 10
volatile char diskTime;

// --- Timer 0 Interrupt  ---------------------------------
//Counts off 1ms and performs necessary tasks
ISR(TIMER0_COMPA_vect) {
	if(btct<59)
		btct++;
	else {
		btct=0;
		buttonstate();
	}
	if(runstate & 0x02){ //if runstate is ACTIVE, the timer should be counting up
		stepms1++;
		stepms2++;
		step2step++;
		if(step2step > (2*1000.0/stepAvg)) {
			stepHz = 0;
			stepAvg = 0;
			pstep1 = 0;
			pstep2 = 0;
			pstep3 = 0;
			pstep4 = 0;
		}
		if (integralCount < 255) {
			integralCount++;
		}
		if (ms < 999) 
			ms++;
		else {
			ms = 0;
			sec++;
	    }
	}
	if (diskTime>0) 
		--diskTime;
   	if (diskTime == 0) {
    	diskTime = diskInterval;
        disk_timerproc();
    }
}

// --- Helper method: button check  ---------------------------------
// --- Checks state of button at D.3
void buttonstate(void) {
	switch(btstate) {
	case NOPUSH: 
		if (BUTTON) 
			btstate = MAYBE;
		break;
	case MAYBE:
		if (BUTTON) {
			bt_fl = 1;
			btstate = PUSH;
			}
		else 
			btstate = NOPUSH;
		break;
	case PUSH:
		if (BUTTON) 
			btstate = PUSH;
		else 
			btstate = MAYBENO;
		break;
	case MAYBENO:
		if (BUTTON) 
			btstate = PUSH;
		else 
			btstate = NOPUSH;
		break;		
	}
}

// --- Data Sampling ----------------------------------
// samples data from adxl and averages every SAMPLE data points
// converts acceleration to m/s^2
void sampleData(void) {
	sample = 0;
	zAccumulator1 = 0;
	zAccumulator2 = 0;
	dt = integralCount; //amount of time between each iteration of sampleData
	total_ms += dt;
	integralCount = 0;
	while (sample < NOISE_AVERAGE) { 
		zAccel1 = readRegister2(XL345_DATAZ0, 4);
		zAccumulator1 += ((int)(zAccel1-zBias1));
		zAccel2 = readRegister2(XL345_DATAZ0, 2);
		zAccumulator2 += ((int)(zAccel2-zBias2));
		sample++;
		_delay_ms(2);
	}
	zAccumulator1 /= NOISE_AVERAGE;
	zAccumulator2 /= NOISE_AVERAGE;
	zAcc1 = zAccumulator1*GAIN_FULL;
	zAcc2 = zAccumulator2*GAIN_FULL;
}

// --- Jerk Threshold Calculation  ---------------------------------
// calculates the absolute value of the change in acceleration
// changes valid time between steps based on jerk 
void jerkCalc(void) {
	jerk1 = pz1 - pz2;
	if (jerk1 < 0) 
		jerk1*= -1;

	// calculations a running average of jerk to smooth data
	jerk1Avg = (jerk1 + pj1 + pj2 + pj3 + pj4 + pj5 + pj6 + pj7)/8.0;
	pj7 = pj6;
	pj6 = pj5;
	pj5 = pj4;
	pj4 = pj3;
	pj3 = pj2;
	pj2 = pj1;
	pj1 = jerk1;

	jerk2 = pzz1 - pzz2;
	if (jerk2 < 0) jerk2*= -1;
	jerk2Avg = (jerk2 + pjj1 + pjj2 + pjj3 + pjj4 + pjj5 + pjj6 + pjj7)/8.0;
	pjj7 = pjj6;
	pjj6 = pjj5; 
	pjj5 = pjj4;
	pjj4 = pjj3;
	pjj3 = pjj2;
	pjj2 = pjj1;
	pjj1 = jerk2;

	// toggles between valid step intervals
	if (jerk1Avg > JERK_THRESH || jerk2Avg > JERK_THRESH) {
		timeSmall = ZSTEP_2_FAST;
		timeLarge = ZSTEP_1_FAST;
	}
	else {
		timeSmall = ZSTEP_2;
		timeLarge = ZSTEP_1;
	}
}

// --- Step Indication  ---------------------------------
// determines the running average of z-axis acceleration
// increments step counter if acceleration and minimum time thresholds pass
void stepCalc(void) {

	//1st accelerometer         
	if(zAcc1 < ZNOISE_MAX && zAcc1 > ZNOISE_MIN) 
		zAcc1 = 0;
	zAccRun1 = (zAcc1 + pz1 + pz2 + pz3 + pz4) / 5.0;
	zAccRun1bar = zAccRun1 *- 1;
	if(zAccRun1 < zAccRun1bar) {
		zAccRun1 *= -1;
		zAccRun1bar *= -1;
	}
	pz4 = pz3;
	pz3 = pz2;
	pz2 = pz1;
	pz1 = zAcc1;

	//2nd accelerometer
	if(zAcc2 < ZNOISE_MAX && zAcc2 > ZNOISE_MIN)
		zAcc2 = 0;
	zAccRun2 = (zAcc2 + pzz1 + pzz2 + pzz3 + pzz4) / 5.0;
	zAccRun2bar = zAccRun2 *- 1;
	if(zAccRun2 < zAccRun2bar) {
		zAccRun2 *= -1;
		zAccRun2bar *= -1;
	}
	pzz4 = pzz3;
	pzz3 = pzz2;
	pzz2 = pzz1;
	pzz1 = zAcc2;

	// tests if acceleration passes thresholds 
 	if (zAcc1 < (zAccRun1bar - 0.3) || zAcc1 > (zAccRun1 + 0.3)) {
		// tests if enough time has passed since last step
		// stepms is time since last step on same foot
		// step2step is time since other foot
		if(stepms1 > timeLarge && step2step > timeSmall) {
        	trtWait(SEM_SHARED);
 				step++;
 			trtSignal(SEM_SHARED);
			stepms1 = 0;
			pstep4 = pstep3;
			pstep3 = pstep2;
			pstep2 = pstep1;	
			// stepHz is the frequency of steps
			stepHz = 1000.0 / step2step;
			pstep1=stepHz;
			// creates a running average of step frequencies
			stepAvg=(stepHz + pstep1 + pstep2 + pstep3 + pstep4) / 5.0;
			step2step=0;
		}
	}

	// same tests for other foot
 	if (zAcc2 < (zAccRun2bar - 0.3) || zAcc2 > (zAccRun2 + 0.3)) {
		if(stepms2 > timeLarge && step2step > timeSmall){
        	trtWait(SEM_SHARED);
 				step++;
 			trtSignal(SEM_SHARED);
 			stepms2 = 0;
			pstep4 = pstep3;
			pstep3 = pstep2;
			pstep2 = pstep1;			
			stepHz = 1000.0/step2step;
			pstep1 = stepHz;
			stepAvg = (stepHz + pstep1 + pstep2 + pstep3 + pstep4) / 5.0;
			step2step=0;
 		}
	}
}

// --- Distance Calculation  ---------------------------------
// determines pace of user and change in distance based on steps per second
void paceDist(void) {
	if(SLOW_FREQ){
		dy = stepAvg * SLOW_WALK*dt;
		currentSpeed = stepAvg * SLOW_WALK;
	}
	else if(MEDIUM_FREQ){
		dy = stepAvg * MEDIUM_WALK*dt;
		currentSpeed = stepAvg * MEDIUM_WALK;
	}
	else if(FAST_FREQ){
		dy = stepAvg * FAST_WALK*dt;
		currentSpeed = stepAvg * FAST_WALK;
	}
	else { //very fast freq
		dy = stepAvg * RUN*dt;
		currentSpeed = stepAvg * RUN;
	}     	
	// updates total distance and current speed
	trtWait(SEM_SHARED);
		latdis += dy;
		curSP = currentSpeed;
	trtSignal(SEM_SHARED);
}

// --- Opening SD File  ---------------------------------
// Attempt to open file on SD card
// If successful, flag is set so that data can be written to file
// defined by the given filePath maintained by fil_obj
void SD_openfile(void) {
	char filePath[20];
	do { //increments file name if file already exists
		snprintf(filePath, 20, "EMS%03d.csv", fileCounter);
		f_err_code = f_open(&fil_obj, filePath, FA_WRITE|FA_CREATE_NEW);
	} while (f_err_code == FR_EXIST && ++fileCounter < 1000);
 	if(f_err_code == FR_OK){ //file successfully opened
		CopyStringtoLCD(LCD_file, 0, 0);
		SD_fl = 1;		
	}
	else 
		CopyStringtoLCD(LCD_fail, 0, 0);	
}

// --- Writing to log file  ---------------------------
void writetoFile(void) {     
	char str[100];
	//sprintf(str,"%8u %3u %5.4f %5.1f %5.4f %5.1f %5.4f %5.4f %5.4f %5.4f, %5.4f\n\r",total_ms,step,zAcc1,zAccumulator1,zAcc2,zAccumulator2,stepHz,stepAvg, zAccRun1, zAccRun2, latdis);
	sprintf(str,"%8u %3u %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f %5.4f\n\r",total_ms, step, latdis, curSP, zAcc1, zAcc2, zAccumulator1, zAccumulator2, jerk1, jerk2, jerk1Avg, jerk2Avg, stepHz, stepAvg);
	f_err_code=f_printf(&fil_obj, str);
	f_err_code=f_sync(&fil_obj);
}

// --- SPI Initialization - ADXL345  -----------------
// ---------------------------------------------------
void xl345_init() { 
	//initialize the 1st adxl
	writeRegister(XL345_DATA_FORMAT, XL345_RANGE_8G | XL345_FULL_RESOLUTION, 4);		
	writeRegister(XL345_BW_RATE, XL345_RATE_800, 4);
	writeRegister(XL345_POWER_CTL, XL345_MEASURE, 4); 

	//initialize the 2nd adxl
	writeRegister(XL345_DATA_FORMAT, XL345_RANGE_8G | XL345_FULL_RESOLUTION, 2);		
	writeRegister(XL345_BW_RATE, XL345_RATE_800, 2);
	writeRegister(XL345_POWER_CTL, XL345_MEASURE, 2); 
}

// --- LCD_init ----------------------------------
void lcd_init(void) 
{
	LCDinit();	
	LCDcursorOFF();
	LCDclr();				
	LCDGotoXY(0,0);
	CopyStringtoLCD(LCD_initialize, 0, 0);
}	

// --- SD_init -------------------------------------------
// --- Initializes SD card in SPI, ensures FATFS works,---
// --- opens log file for writing data out----------------
void SD_init(void) 
{
	status = disk_initialize(0); // 0 status means disk initialized
	f_err_code = f_mount(0, &FATFS_Obj); // 0 err code means disk mounted
	if(status | f_err_code)
		CopyStringtoLCD(LCD_fail, 0, 0);
}	

// --- Initialization (acceleration parameters) --------------------------
// calculates a calibration value to 0 acceleration data based on current orientation of accelerometer
void calc_init() {
    CopyStringtoLCD(LCD_calibrate, 0, 1);

	// Sets up offset bias for z-axis of 1st accelerometer
    calibrationCount = 0;
	zAccumulator1 = 0;
    while (calibrationCount < CALIBRATION_COUNT) {
        //Make sure the accelerometer has had enough time to take new sample
		zAccel1 = readRegister2(XL345_DATAZ0, 4);
        zAccumulator1 += zAccel1;
        calibrationCount++; 
        _delay_ms(10);
    }
    zBias1 = (unsigned int)(zAccumulator1 / CALIBRATION_COUNT);
	
	// Sets up offset bias for z-axis of 2nd accelerometer
	calibrationCount = 0;
	zAccumulator2 = 0;
    while (calibrationCount < CALIBRATION_COUNT) {
		zAccel2 = readRegister2(XL345_DATAZ0, 2);
        zAccumulator2 += zAccel2;
        calibrationCount++;  
        _delay_ms(10);
    }
    zBias2 = (unsigned int)(zAccumulator2 / CALIBRATION_COUNT);	
}

// --- Initialize and zero out parameters --------------------------
void zeroVars (void) {
	trtWait(SEM_SHARED);
		curSP = 0;
		step = 0;
		latdis = 0;
	trtSignal(SEM_SHARED);
					
	zAcc1 = 0;
	zAcc2 = 0;
	zAccumulator1 = 0;
	zAccumulator2 = 0;
	dt = 0;
	total_ms=0;
	ms = 0;
	sec = 0;
	stepms1 = 0;
	stepms2 = 0;
	stepAvg = 0;
	step2step = 0;
	window = 0;
	stepHz = 0;
	pstep1 = 0;
	pstep2 = 0;
	pstep3 = 0;
	pstep4 = 0;
	pz1 = 0;
	pz2 = 0;
	pz3 = 0;
	pz4 = 0;
	pzz1 = 0;
	pzz2 = 0;
	pzz3 = 0;
	pzz4 = 0;
	zAccRun1 = 0;
	zAccRun2 = 0;
	zAccRun1bar = 0;
	zAccRun2bar = 0;
	jerk1 = 0;
	jerk1Avg = 0;
	pj1 = 0;
	pj2 = 0;
	pj3 = 0;
	pj4 = 0;
	pj5 = 0;
	pj6 = 0;
	pj7 = 0;
	jerk2 = 0;
	jerk2Avg = 0;
	pjj1 = 0;
	pjj2 = 0;
	pjj3 = 0;
	pjj4 = 0;
	pjj5 = 0;
	pjj6 = 0;
	pjj7 = 0;
	timeSmall = ZSTEP_2;
	timeLarge = ZSTEP_1;
}


// --- TASK 1 - Data acquisition ----------------------------------
// sample data, then calculate change in location and speed
// button press either closes current logging session or begins initialization
void stepData(void* args){
	uint32_t rel,dead;
	zeroVars();
	if(SD_fl)
		writetoFile();
	while(1) {
		// if button is pushed and not yet initialized
		if(btstate==PUSH && bt_fl == 1) { 
			bt_fl = 0;
			LCDclr();
			switch(runstate) {
				case(IDLE):
					runstate = INIT; //go immediately from idle to init after button press
				break;
				case(ACTIVE): //occurs when ending current session
					runstate = IDLE; //turn back to idle state
					zeroVars();
					if(SD_fl){ //close SD card to log data
						f_sync(&fil_obj);
						f_err_code = f_close(&fil_obj);
					}
					trtWait(SEM_LCD_SHARED);
					if(f_err_code==FR_OK) {
						CopyStringtoLCD(LCD_close, 0, 0);
					}
					else {
						CopyStringtoLCD(LCD_fail, 0, 0);
					}
					CopyStringtoLCD(LCD_idle, 0, 0);
					trtSignal(SEM_LCD_SHARED);
				break;
				default:
				break;
			}
		}
		switch(runstate) { //normal state if no button press
			case(IDLE):
			break;
			case(INIT): //initializes all parameters
				LCDclr();
				calc_init();
				SD_openfile();
				if(SD_fl)
					writetoFile();
				trtWait(SEM_LCD_SHARED);
				CopyStringtoLCD(LCD_time, 0, 0);
				trtSignal(SEM_LCD_SHARED);
				runstate=ACTIVE;
			break;
			case(ACTIVE):
				sampleData();
				stepCalc();
				paceDist();
				jerkCalc();
				if(SD_fl)	
					writetoFile();
			break;
		}		

		rel = trtCurrentTime() + SECONDS2TICKS(0.02);
		dead = trtCurrentTime() + SECONDS2TICKS(0.32);
		trtSleepUntil(rel, dead);
	}
}

// --- Task 2 - LCD Screen ----------------------------------
// Update the LCD screen
void updateLCD(void* args){
	uint32_t rel,dead;
    uint8_t sec_60, min, sectotal;
    double ydis_lcd, curSP_lcd;
    uint16_t step_lcd;
	while(1) {
		if(runstate == ACTIVE) {
			//variables to be displayed
	       trtWait(SEM_SHARED);
		       ydis_lcd = latdis;
		       step_lcd = step;
			   curSP_lcd = curSP;
	       trtSignal(SEM_SHARED);
	       sectotal = sec;
	       sec_60 = sectotal % 60;
	       min = sectotal / 60;
	       
			trtWait(SEM_LCD_SHARED);
		   //print out time
		   LCDGotoXY(6,0);
		   sprintf(lcd_buffer, "%02u:%02u",min,sec_60);
		   LCDstring(lcd_buffer, strlen(lcd_buffer));
		   //print steps
		   if(sec_60%6<2) { //sec_60%6 is 0 or 1
	       		CopyStringtoLCD(LCD_step, 0, 1);
		     	LCDGotoXY(6,1);
	  	   		sprintf(lcd_buffer, "%3u",step_lcd);
		   		LCDstring(lcd_buffer, strlen(lcd_buffer));
	    	}
			//print distance
		   else if (sec_60%6<4) {//sec_60%6 is 2 or 3
		   		CopyStringtoLCD(LCD_dist, 0, 1);
		   		LCDGotoXY(10,1);
	            dtostrf(ydis_lcd,5,3,lcd_buffer);
		        LCDstring(lcd_buffer, strlen(lcd_buffer));	
	    	}
			//print current speed
		   else { //sec_60%6 is 4 or 5
		   		CopyStringtoLCD(LCD_avsp, 0, 1);
	          	LCDGotoXY(10,1);
		      	dtostrf(curSP_lcd,5,3,lcd_buffer);
		      	LCDstring(lcd_buffer, strlen(lcd_buffer));	
			}
			trtSignal(SEM_LCD_SHARED);
		}    
    	rel = trtCurrentTime() + SECONDS2TICKS(0.25);
		dead = trtCurrentTime() + SECONDS2TICKS(0.75);
		trtSleepUntil(rel, dead);
	}
}

// --- Main Program ----------------------------------
// initialize everything, then keep the tasker running
int main(void) {
	btct = 0;
	SD_fl = 0;
	bt_fl = 0;
	fileCounter = 0;
	btstate = NOPUSH;
	DDRD = (1<<PORTD2) | (0<<PORTD7);	// PORT D.2 is an ouput, D.7 is a button input

	TIMSK0 = 2;		//turn on timer 0 cmp match ISR 
	OCR0A = 249;  	//set the compare reg to 250 time ticks
	TCCR0A = 0b00000010; // turn on clear-on-match
	TCCR0B = 0b00000011;	// clock prescalar to 64

	runstate = IDLE;
	run_fl = 0;
	sei(); 
	
	_delay_ms(2000); //wait 2 seconds for MCU to warm up
	trt_uart_init();
	stdout = stdin = stderr = &uart_str;
	_delay_ms(20);

	lcd_init();
	SPI_init();
	xl345_init();
	SD_init();

	_delay_ms(100);
	CopyStringtoLCD(LCD_idle, 0, 0);
	trtInitKernel(500); // 500 bytes for the idle task stack

	// --- create semaphores ----------
	// You must creat the first two semaphores if you use the uart
	trtCreateSemaphore(SEM_RX_ISR_SIGNAL, 0) ; // uart receive ISR semaphore
	trtCreateSemaphore(SEM_STRING_DONE,0) ;  // user typed <enter>
	// variable protection
	trtCreateSemaphore(SEM_SHARED, 1) ; // protect shared variables
	trtCreateSemaphore(SEM_LCD_SHARED, 1) ; // protect shared variables
	// --- create tasks  ----------------
	trtCreateTask(stepData, 1500, SECONDS2TICKS(0.2), SECONDS2TICKS(0.3), &(args[0]));
	trtCreateTask(updateLCD, 1500, SECONDS2TICKS(1), SECONDS2TICKS(1.3), &(args[0]));

	while (1) { // indicates program is still running
		led = led ^ 1 ;
	  	PORTD = (led<<PORTD2) ;
		_delay_ms(500) ;
	}
} 
