/*Iniitializes the SPI digital interface with the accelerometer and contains read and write methods*/

#include "spi_init.h"
#include "adxl345.h"
#include <avr/pgmspace.h>
#include <string.h>
#include <stdio.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#define SET(U, N) ((void)((U) |= 1u << (N)))
#define CLR(U, N) ((void)((U) &= ~(1u << (N))))

unsigned char junk, data, dataLSB, dataMSB;

/*
 * Initializes the SPI interface
 */
void SPI_init(void) {
	//B.4 CS
	//B.5 MOSI SDA
	//B.6 MISO SDO
	//B.7 SCLK
	//set up i/o data direction   
	DDRB |= (1<<DDB2) | (1<<DDB3) | (1<<DDB4) | (1<<DDB5) | (1<<DDB7) | (0<<DDB6); 
	//init CS high
	SET(PORTB,2);
	SET(PORTB,3); 
	SET(PORTB,4); 
	//Interrupt Off| SPI On | MSB First | Master Mode | Polarity | Phase | Fosc/4
	SPCR = (1 << SPE) | (1 << MSTR) | (1 << CPOL) | (1 << CPHA) | (0 << SPR1) | (0 << SPR0);
	//SPSR = not 2x speed
}

//Write single registers to accelerometer
void writeRegister(unsigned char address, unsigned char value, unsigned char pin) {
	CLR(PORTB,pin);							//CS = low

	SPDR = address;							//send address out
	while(!(SPSR & (1 << SPIF)));  			//wait for tx buf empty
		junk = SPDR;						//receive junk

	SPDR = value;							//write value
	while(!(SPSR & (1 << SPIF)));			//wait for tx buf empty
		junk = SPDR;						//receive junk
	
	SET(PORTB,pin);							//CS = high	
}

//Read from single data register from accelerometer
unsigned char readRegister(unsigned char start_address, unsigned char pin) {
	//Since we're performing a read operation, the most significant bit of the register address should be set.
 	unsigned char address = 0x80 | start_address;

	CLR(PORTB,pin);							//CS = low
	SPDR = address;							//send address out
	while(!(SPSR & (1 << SPIF)));  			//wait for tx buf empty
		junk = SPDR;						//receive junk

	SPDR = junk;							//write junk value
	while(!(SPSR & (1 << SPIF)));			//wait for tx buf empty
		data = SPDR;						//receive data
	SET(PORTB,pin);							//CS = high	
	return data;
}

//Read from single data register from accelerometer
unsigned int readRegister2(unsigned char start_address, unsigned char pin) {
	//For multiple read, 0x40 is set also
 	unsigned char address = 0x80 | 0x40 | start_address;

	CLR(PORTB,pin);							//CS = low

	SPDR = address;							//send address out
	while(!(SPSR & (1 << SPIF)));  			//wait for tx buf empty
		junk = SPDR;						//receive junk

	SPDR = junk;							//write junk value
	while(!(SPSR & (1 << SPIF)));			//wait for tx buf empty
		dataLSB = SPDR;						//receive data

	SPDR = junk;							
	while(!(SPSR & (1 << SPIF)));			
		dataMSB = SPDR;						//receive second data

	SET(PORTB,pin);							//CS = high	
	return ((dataMSB << 8) | dataLSB);
}

